@@ -19,14 +19,12 @@
 
 #include "xlayout.hpp"
 #include "xshape.hpp"
+#include "xtensor_forward.hpp"
 #include "xutils.hpp"
 
 namespace xt
 {
 
-    template <class E>
-    class xshared_expression;
-
     /***************************
      * xexpression declaration *
      ***************************/
@@ -136,6 +134,36 @@ namespace xt
     template <class... E>
     using has_xexpression = xtl::disjunction<is_xexpression<E>...>;
 
+    /***********************
+     * evaluation_strategy *
+     ***********************/
+
+    namespace evaluation_strategy
+    {
+        struct base
+        {
+        };
+
+        struct immediate : base
+        {
+        };
+        
+        struct lazy : base
+        {
+        };
+        
+        /*
+        struct cached
+        {
+        };
+        */
+    }
+
+    template <class T>
+    struct is_evaluation_strategy : std::is_base_of<evaluation_strategy::base, std::decay_t<T>>
+    {
+    };
+
     /************
      * xclosure *
      ************/
@@ -143,6 +171,9 @@ namespace xt
     template <class T>
     class xscalar;
 
+    template <class E>
+    class xshared_expression;
+
     template <class E, class EN = void>
     struct xclosure
     {
@@ -210,6 +241,48 @@ namespace xt
     template <class E>
     using xvalue_type_t = typename xvalue_type<E>::type;
 
+    /***********************************
+     * temporary_type_t implementation *
+     ***********************************/
+
+    namespace detail
+    {
+        template <class S>
+        struct xtype_for_shape
+        {
+            template <class T, layout_type L>
+            using type = xarray<T, L>;
+        };
+
+#if defined(__GNUC__) && (__GNUC__ > 6)
+#if __cplusplus == 201703L 
+        template <template <class, std::size_t, class, bool> class S, class X, std::size_t N, class A, bool Init>
+        struct xtype_for_shape<S<X, N, A, Init>>
+        {
+            template <class T, layout_type L>
+            using type = xarray<T, L>;
+        };
+#endif // __cplusplus == 201703L
+#endif // __GNUC__ && (__GNUC__ > 6)
+
+        template <template <class, std::size_t> class S, class X, std::size_t N>
+        struct xtype_for_shape<S<X, N>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor<T, N, L>;
+        };
+
+        template <template <std::size_t...> class S, std::size_t... X>
+        struct xtype_for_shape<S<X...>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor_fixed<T, xshape<X...>, L>;
+        };
+
+        template <class T, class S, layout_type L>
+        using temporary_type_t = typename xtype_for_shape<S>::template type<T, L>;
+    }
+ 
     /*************************
      * expression tag system *
      *************************/
