@@ -325,31 +325,29 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
 
 #define XTENSOR_REDUCER_FUNCTION(NAME, FUNCTOR, RESULT_TYPE)                                                      \
     template <class T = void, class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,                            \
-              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value        \
-                        && !std::is_integral<X>::value, int>>                                                     \
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>, xtl::negation<std::is_integral<X>>)>         \
     inline auto NAME(E&& e, X&& axes, EVS es = EVS())                                                             \
     {                                                                                                             \
         using result_type = std::conditional_t<std::is_same<T, void>::value, RESULT_TYPE, T>;                     \
         using functor_type = FUNCTOR<result_type>;                                                                \
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e),                                  \
+        return xt::reduce(make_xreducer_functor(functor_type()), std::forward<E>(e),                              \
                       std::forward<X>(axes), es);                                                                 \
     }                                                                                                             \
                                                                                                                   \
     template <class T = void, class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,                            \
-              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value        \
-                        && std::is_integral<X>::value, int>>                                                      \
-    inline auto NAME(E&& e, X axis, EVS es = EVS())                                                                \
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>, std::is_integral<X>)>                        \
+    inline auto NAME(E&& e, X axis, EVS es = EVS())                                                               \
     {                                                                                                             \
-        return NAME(std::forward<E>(e), {axis}, es);                                                               \
+        return NAME(std::forward<E>(e), {axis}, es);                                                              \
     }                                                                                                             \
                                                                                                                   \
     template <class T = void, class E, class EVS = DEFAULT_STRATEGY_REDUCERS,                                     \
-              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, EVS>::value, int>>              \
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>                                                          \
     inline auto NAME(E&& e, EVS es = EVS())                                                                       \
     {                                                                                                             \
         using result_type = std::conditional_t<std::is_same<T, void>::value, RESULT_TYPE, T>;                     \
         using functor_type = FUNCTOR<result_type>;                                                                \
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), es);                             \
+        return xt::reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), es);                         \
     }
 
 #define XTENSOR_OLD_CLANG_REDUCER(NAME, FUNCTOR, RESULT_TYPE)                                                     \
@@ -358,7 +356,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     {                                                                                                             \
         using result_type = std::conditional_t<std::is_same<T, void>::value, RESULT_TYPE, T>;                     \
         using functor_type = FUNCTOR<result_type>;                                                                \
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes, es);                       \
+        return xt::reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes, es);                   \
     }
 
 #define XTENSOR_MODERN_CLANG_REDUCER(NAME, FUNCTOR, RESULT_TYPE)                                                  \
@@ -367,7 +365,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     {                                                                                                             \
         using result_type = std::conditional_t<std::is_same<T, void>::value, RESULT_TYPE, T>;                     \
         using functor_type = FUNCTOR<result_type>;                                                                \
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes, es);                       \
+        return xt::reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes, es);                   \
     }
 
     /*******************
@@ -884,7 +882,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
                 return m_lambda(args...);
             }
 
-            template <class... T, class = std::enable_if_t<detail::supports<F(T...)>::value, int>>
+            template <class... T, XTL_REQUIRES(detail::supports<F(T...)>)>
             auto simd_apply(T... args) const
             {
                 return m_lambda(args...);
@@ -1736,7 +1734,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
      * @return an \ref xexpression
      */
     template <class T = void, class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value>>
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>)>
     inline auto mean(E&& e, X&& axes, EVS es = EVS())
     {
         using value_type = typename std::conditional_t<std::is_same<T, void>::value, double, T>;
@@ -1748,7 +1746,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     }
 
     template <class T = void, class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, std::decay_t<EVS>>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto mean(E&& e, EVS es = EVS())
     {
         using value_type = typename std::conditional_t<std::is_same<T, void>::value, double, T>;
@@ -1789,8 +1787,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
      * @sa mean
      */
     template <class E, class W, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, EVS>::value>,
-              XTENSOR_REQUIRE<!std::is_integral<X>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>, xtl::negation<std::is_integral<X>>)>
     inline auto average(E&& e, W&& weights, X&& axes, EVS ev = EVS())
     {
         xindex_type_t<typename std::decay_t<E>::shape_type> broadcast_shape;
@@ -1839,7 +1836,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
 #endif
 
     template <class E, class W, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, EVS>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto average(E&& e, W&& weights, EVS ev = EVS())
     {
         if (weights.dimension() != e.dimension() || !std::equal(weights.shape().begin(), weights.shape().end(), e.shape().begin()))
@@ -1870,7 +1867,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     }
 
     template <class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, EVS>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto variance(E&& e, EVS es = EVS())
     {
         decltype(auto) sc = detail::shared_forward<E>(e);
@@ -1878,7 +1875,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     }
 
     template <class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, EVS>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto stddev(E&& e, EVS es = EVS())
     {
         return sqrt(variance(std::forward<E>(e), es));
@@ -1902,7 +1899,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
      * @sa stddev, mean
      */
     template <class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value>>
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>)>
     inline auto variance(E&& e, X&& axes, EVS es = EVS())
     {
         decltype(auto) sc = detail::shared_forward<E>(e);
@@ -1938,7 +1935,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
      * @sa variance, mean
      */
     template <class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value>>
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>)>
     inline auto stddev(E&& e, X&& axes, EVS es = EVS())
     {
         return sqrt(variance(std::forward<E>(e), std::forward<X>(axes), es));
@@ -1948,25 +1945,33 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     template <class E, class A, std::size_t N, class EVS = DEFAULT_STRATEGY_REDUCERS>
     inline auto stddev(E&& e, const A (&axes)[N], EVS es = EVS())
     {
-        return stddev(std::forward<E>(e), xtl::forward_sequence<std::array<std::size_t, N>>(axes), es);
+        return stddev(std::forward<E>(e),
+                      xtl::forward_sequence<std::array<std::size_t, N>, decltype(axes)>(axes),
+                      es);
     }
 
     template <class E, class A, std::size_t N, class EVS = DEFAULT_STRATEGY_REDUCERS>
     inline auto variance(E&& e, const A (&axes)[N], EVS es = EVS())
     {
-        return variance(std::forward<E>(e), xtl::forward_sequence<std::array<std::size_t, N>>(axes), es);
+        return variance(std::forward<E>(e),
+                        xtl::forward_sequence<std::array<std::size_t, N>, decltype(axes)>(axes),
+                        es);
     }
 #else
     template <class E, class A, class EVS = DEFAULT_STRATEGY_REDUCERS>
     inline auto stddev(E&& e, std::initializer_list<A> axes, EVS es = EVS())
     {
-        return stddev(std::forward<E>(e), xtl::forward_sequence<dynamic_shape<std::size_t>>(axes), es);
+        return stddev(std::forward<E>(e),
+                      xtl::forward_sequence<dynamic_shape<std::size_t>, decltype(axes)>(axes),
+                      es);
     }
 
     template <class E, class A, class EVS = DEFAULT_STRATEGY_REDUCERS>
     inline auto variance(E&& e, std::initializer_list<A> axes, EVS es = EVS())
     {
-        return variance(std::forward<E>(e), xtl::forward_sequence<dynamic_shape<std::size_t>>(axes), es);
+        return variance(std::forward<E>(e),
+                        xtl::forward_sequence<dynamic_shape<std::size_t>, decltype(axes)>(axes),
+                        es);
     }
 #endif
     /**
@@ -1980,7 +1985,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
      *         and second element represent the minimum and maximum respectively
      */
     template <class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              XTENSOR_REQUIRE<std::is_base_of<evaluation_strategy::base, EVS>::value>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto minmax(E&& e, EVS es = EVS())
     {
         using std::min;
@@ -2001,7 +2006,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
             r[1] = (max)(r[1], s[1]);
             return r;
         };
-        return reduce(make_xreducer_functor(std::move(reduce_func),
+        return xt::reduce(make_xreducer_functor(std::move(reduce_func),
                                             std::move(init_func),
                                             std::move(merge_func)),
                       std::forward<E>(e), arange(e.dimension()), es);
@@ -2147,24 +2152,24 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
 
 #define XTENSOR_NAN_REDUCER_FUNCTION(NAME, FUNCTOR, RESULT_TYPE, NAN)                                             \
     template <class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,                                            \
-              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value, int>> \
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>)>                                              \
     inline auto NAME(E&& e, X&& axes, EVS es = EVS())                                                             \
     {                                                                                                             \
         using result_type = RESULT_TYPE;                                                                          \
         using functor_type = FUNCTOR<result_type>;                                                                \
         using init_functor_type = detail::nan_init<result_type, NAN>;                                             \
-        return reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e),             \
+        return xt::reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e),         \
                       std::forward<X>(axes), es);                                                                 \
     }                                                                                                             \
                                                                                                                   \
     template <class E, class EVS = DEFAULT_STRATEGY_REDUCERS,                                                     \
-              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, EVS>::value, int>>              \
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>                                                          \
     inline auto NAME(E&& e, EVS es = EVS())                                                                       \
     {                                                                                                             \
         using result_type = RESULT_TYPE;                                                                          \
         using functor_type = FUNCTOR<result_type>;                                                                \
         using init_functor_type = detail::nan_init<result_type, NAN>;                                             \
-        return reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), es);        \
+        return xt::reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), es);    \
     }
 
 #define OLD_CLANG_NAN_REDUCER(NAME, FUNCTOR, RESULT_TYPE, NAN)                                                       \
@@ -2174,7 +2179,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
             using result_type = RESULT_TYPE;                                                                         \
             using functor_type = FUNCTOR<result_type>;                                                               \
             using init_functor_type = detail::nan_init<result_type, NAN>;                                            \
-            return reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), axes, es); \
+            return xt::reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), axes, es); \
         }
 
 #define MODERN_CLANG_NAN_REDUCER(NAME, FUNCTOR, RESULT_TYPE, NAN)                                                 \
@@ -2184,7 +2189,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
         using result_type = RESULT_TYPE;                                                                          \
         using functor_type = FUNCTOR<result_type>;                                                                \
         using init_functor_type = detail::nan_init<result_type, NAN>;                                             \
-        return reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), axes, es);  \
+        return xt::reduce(make_xreducer_functor(functor_type(), init_functor_type()), std::forward<E>(e), axes, es);  \
     }
 
     /**
@@ -2242,27 +2247,25 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     auto merge_func = std::plus<result_type>();                                 \
 
     template <class E, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, EVS>::value, int>>
+              XTL_REQUIRES(is_evaluation_strategy<EVS>)>
     inline auto count_nonzero(E&& e, EVS es = EVS())
     {
         COUNT_NON_ZEROS_CONTENT;
-        return reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
+        return xt::reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
                       std::forward<E>(e), es);
     }
 
     template <class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, X>::value
-                                       && !std::is_integral<X>::value, int>>
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>, xtl::negation<std::is_integral<X>>)>
     inline auto count_nonzero(E&& e, X&& axes, EVS es = EVS())
     {
         COUNT_NON_ZEROS_CONTENT;
-        return reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
+        return xt::reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
                       std::forward<E>(e), std::forward<X>(axes), es);
     }
 
     template <class E, class X, class EVS = DEFAULT_STRATEGY_REDUCERS,
-            class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, X>::value
-                                     && std::is_integral<X>::value, int>>
+              XTL_REQUIRES(xtl::negation<is_evaluation_strategy<X>>, std::is_integral<X>)>
     inline auto count_nonzero(E&& e, X axis, EVS es = EVS())
     {
         return count_nonzero(std::forward<E>(e), {axis}, es);
@@ -2273,7 +2276,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     inline auto count_nonzero(E&& e, std::initializer_list<I> axes, EVS es = EVS())
     {
         COUNT_NON_ZEROS_CONTENT;
-        return reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
+        return xt::reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
                       std::forward<E>(e), axes, es);
     }
 #else
@@ -2281,7 +2284,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     inline auto count_nonzero(E&& e, const I (&axes)[N], EVS es = EVS())
     {
         COUNT_NON_ZEROS_CONTENT;
-        return reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
+        return xt::reduce(make_xreducer_functor(std::move(reduce_fct), std::move(init_fct), std::move(merge_func)),
                       std::forward<E>(e), axes, es);
     }
 #endif
