@@ -732,7 +732,7 @@ namespace xt
 
         xview_stepper() = default;
         xview_stepper(view_type* view, substepper_type it,
-                      size_type offset, bool end = false, layout_type l = XTENSOR_DEFAULT_LAYOUT);
+                      size_type offset, bool end = false, layout_type l = XTENSOR_DEFAULT_TRAVERSAL);
 
         reference operator*() const;
 
@@ -786,7 +786,7 @@ namespace xt
     template <std::size_t... I, class... S>
     struct xview_shape_type<fixed_shape<I...>, S...>
     {
-        using type = typename xview_shape_type<std::array<std::size_t, sizeof...(I)>>::type;
+        using type = typename xview_shape_type<std::array<std::size_t, sizeof...(I)>, S...>::type;
     };
 
     /************************
